@@ -14,6 +14,9 @@
 
 import sys, os
 
+# If your extensions are in another directory, add it here. If the directory
+# is relative to the documentation root, use os.path.abspath to make it
+# absolute, like shown here.
 parent = os.path.dirname(os.path.dirname(__file__))
 sys.path.append(os.path.abspath(parent))
 wd = os.getcwd()
@@ -25,17 +28,13 @@ for item in os.listdir(parent):
     if item.endswith('.egg'):
         sys.path.append(os.path.join(parent, item))
 
-# If your extensions are in another directory, add it here. If the
-# directory is relative to the documentation root, use os.path.abspath to
-# make it absolute, like shown here.
-#sys.path.append(os.path.abspath('some/directory'))
-
 # General configuration
 # ---------------------
 
 # Add any Sphinx extension module names here, as strings. They can be
 # extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
-extensions = ['sphinx.ext.autodoc', 'repoze.sphinx.autointerface']
+extensions = ['sphinx.ext.autodoc',
+              'repoze.sphinx.autointerface']
 
 # Add any paths that contain templates here, relative to this directory.
 templates_path = ['.templates']
@@ -54,9 +53,9 @@ copyright = '2008, Repoze Developers <repoze-dev@lists.repoze.org>'
 # other places throughout the built documents.
 #
 # The short X.Y version.
-version = '0.4'
+version = '0.5'
 # The full version, including alpha/beta/rc tags.
-release = '0.4'
+release = '0.5'
 
 # There are two options for replacing |today|: either, you set today to
 # some non-false value, then it is used:
