@@ -40,11 +40,13 @@ class TestWorkflowDirective(unittest.TestCase):
             pass
         directive = self._makeOne(initial_state='public', type='security',
                                   content_types=(IDummy, IDummy2))
-        directive.states = [ DummyState('private', a=1),
-                             DummyState('public', b=2) ]
-        directive.transitions = [ DummyTransition('make_public'),
-                                  DummyTransition('make_private'),
-                                  ]
+        directive.states = [DummyState('private', title='Private', a=1),
+                            DummyState('public', b=2)
+                           ]
+        directive.transitions = [DummyTransition('make_public'),
+                                 DummyTransition('make_private',
+                                                 title='Retract'),
+                                ]
         directive.after()
         actions = directive.context.actions
         self.assertEqual(len(actions), 2)
@@ -67,18 +69,26 @@ class TestWorkflowDirective(unittest.TestCase):
         self.assertEqual(wf_dict['elector'], None)
         self.assertEqual(wf_dict['workflow'].__class__, Workflow)
         workflow = wf_dict['workflow']
+        self.assertEqual(
+            workflow._state_data,
+            {'public':
+                {'callback': None, 'title': 'public', 'b': 2},
+             'private':
+                {'callback': None, 'title': 'Private', 'a': 1},
+             })
         self.assertEqual(
             workflow._transition_data,
             {'make_public':
              {'from_state': 'private', 'callback': None,
-              'name': 'make_public', 'to_state': 'public',
-              'permission':None},
+              'guards': [], 'name': 'make_public', 
+              'to_state': 'public', 'permission':None, 
+              'title': 'make_public'},
              'make_private':
              {'from_state': 'private', 'callback': None,
-              'name': 'make_private', 'to_state': 'public',
-              'permission':None},
-             }
-            )
+              'guards': [], 'name': 'make_private', 
+              'to_state': 'public', 'permission':None, 
+              'title': 'Retract'},
+             })
         self.assertEqual(workflow.initial_state, 'public')
 
         action = actions[1]
@@ -99,20 +109,42 @@ class TestWorkflowDirective(unittest.TestCase):
         self.assertEqual(wf_dict['elector'], None)
         self.assertEqual(wf_dict['workflow'].__class__, Workflow)
         workflow = wf_dict['workflow']
+        self.assertEqual(
+            workflow._state_data,
+            {'public':
+                {'callback': None, 'title': 'public', 'b': 2},
+             'private':
+                {'callback': None, 'title': 'Private', 'a': 1},
+             })
         self.assertEqual(
             workflow._transition_data,
             {'make_public':
              {'from_state': 'private', 'callback': None,
-              'name': 'make_public', 'to_state': 'public',
-              'permission':None},
+              'guards': [], 'name': 'make_public', 
+              'to_state': 'public', 'permission':None, 
+              'title': 'make_public'},
              'make_private':
              {'from_state': 'private', 'callback': None,
-              'name': 'make_private', 'to_state': 'public',
-              'permission':None},
+              'guards': [], 'name': 'make_private', 
+              'to_state': 'public', 'permission':None, 
+              'title': 'Retract'},
              }
             )
         self.assertEqual(workflow.initial_state, 'public')
 
+    def test_after_warns_if_no_content_types(self):
+        import warnings
+        directive = self._makeOne(initial_state='public', type='security')
+        directive.states = [ DummyState('private', a=1),
+                             DummyState('public', b=2) ]
+        directive.transitions = [ DummyTransition('make_public'),
+                                  DummyTransition('make_private'),
+                                  ]
+        with warnings.catch_warnings(record=True) as log:
+            directive.after()
+        self .assertEqual(len(log), 1)
+        self .assertEqual(log[0].category, UserWarning)
+
     def test_after_raises_error_during_transition_add(self):
         from zope.interface import Interface
         from zope.configuration.exceptions import ConfigurationError
@@ -228,6 +260,18 @@ class TestKeyValuePair(unittest.TestCase):
         self._callFUT(context, 'key', 'value')
         self.assertEqual(context.context.extras, {'key':'value'})
 
+class TestGuard(unittest.TestCase):
+    def _callFUT(self, context, func):
+        from repoze.workflow.zcml import guard_function
+        guard_function(context, func)
+
+    def test_it_no_extras(self):
+        context = DummyTransition('dummy')
+        def example(context, transition):  # pragma: NO COVER
+            return None
+        self._callFUT(context, example)
+        self.assertEqual(context.guards, [example])
+
 class TestAlias(unittest.TestCase):
     def _callFUT(self, context, name):
         from repoze.workflow.zcml import alias
@@ -247,15 +291,17 @@ class TestFixtureApp(unittest.TestCase):
         cleanUp()
 
     def test_execute_actions(self):
+        from zope.configuration import xmlconfig
+        from zope.component import getSiteManager
         from repoze.workflow.interfaces import IWorkflowList
         from repoze.workflow.workflow import Workflow
         from repoze.workflow.tests.fixtures.dummy import callback
         import repoze.workflow.tests.fixtures as package
+        from repoze.workflow.tests.fixtures import dummy
         from repoze.workflow.tests.fixtures.dummy import IContent
         from repoze.workflow.tests.fixtures.dummy import elector
         from repoze.workflow.tests.fixtures.dummy import has_permission
-        from zope.configuration import xmlconfig
-        from zope.component import getSiteManager
+        from repoze.workflow._compat import text_ as _u
         xmlconfig.file('configure.zcml', package, execute=True)
         sm = getSiteManager()
         wf_list = sm.adapters.lookup((IContent,),
@@ -269,33 +315,51 @@ class TestFixtureApp(unittest.TestCase):
         self.assertEqual(workflow.description, 'The workflow which is of the '
                          'testing fixtures package')
         self.assertEqual(workflow.permission_checker, has_permission)
-        self.assertEqual(
-            workflow._state_order,
-            ['private', 'public'],
-            )
         self.assertEqual(
             workflow._state_aliases,
             {'supersecret':'private'},
             )
         self.assertEqual(
             workflow._state_data,
-            {u'public': {'callback':callback,
-                         'description': u'Everybody can see it',
-                         'title': u'Public'},
-             u'private': {'callback':callback,
-                          'description': u'Nobody can see it',
-                          'title': u'Private'}},
-            )
+            {_u('public'):
+                {'callback':callback,
+                 'description': _u('Everybody can see it'),
+                 'title': _u('Public'),
+                },
+             _u('private'):
+                {'callback':callback,
+                 'description': _u('Nobody can see it'),
+                 'title': _u('Private'),
+                },
+            })
         transitions = workflow._transition_data
-        self.assertEqual(len(transitions), 2)
+        self.assertEqual(len(transitions), 3)
         self.assertEqual(transitions['private_to_public'],
-            {'from_state': u'private', 'callback': callback,
-             'name': u'private_to_public', 'to_state': u'public',
-             'permission':'moderate'}),
+            {'from_state': _u('private'),
+             'callback': callback,
+             'guards': [],
+             'name': _u('private_to_public'),
+             'to_state': _u('public'),
+             'permission':'moderate',
+             'title': 'private_to_public',
+            }),
+        self.assertEqual(transitions['unavailable_public_to_private'],
+            {'from_state': _u('public'),
+             'callback': callback,
+             'guards': [dummy.never],
+             'name': _u('unavailable_public_to_private'),
+             'to_state': _u('private'),
+             'permission':_u('moderate'),
+             'title': _u('unavailable_public_to_private'),
+            }),
         self.assertEqual(transitions['public_to_private'],
-             {'from_state': u'public', 'callback': callback,
-              'name': 'public_to_private', 'to_state': u'private',
-              'permission':'moderate',}
+             {'from_state': _u('public'),
+              'callback': callback,
+              'guards': [],
+              'name': 'public_to_private',
+              'to_state': _u('private'),
+              'permission':'moderate',
+              'title': 'public_to_private'}
             )
 
 class TestRegisterWorkflow(unittest.TestCase):
@@ -341,7 +405,7 @@ class TestRegisterWorkflow(unittest.TestCase):
         from zope.component import getSiteManager
         from zope.interface import providedBy
         from zope.interface import Interface
-        class Foo:
+        class Foo(object):
             pass
         class IBar(Interface):
             pass
@@ -361,20 +425,21 @@ class DummyContext:
         self.__dict__.update(kw)
 
 class DummyState:
-    def __init__(self, name, title=None, callback=None, aliases=(), **extras):
+    def __init__(self, name, callback=None, aliases=(), title=None, **extras):
         self.name = name
+        self.title = title
         self.callback = callback
         self.extras = extras
         self.aliases = aliases
         
 class DummyTransition:
     def __init__(self, name, from_state='private', to_state='public',
-                 callback=None, permission=None, **extras):
+                 callback=None, permission=None, title=None, **extras):
         self.name = name
         self.from_state = from_state
         self.to_state = to_state
         self.callback = callback
         self.permission = permission
+        self.title = title
         self.extras = extras
-
-                  
+        self.guards = []
